#!/usr/bin/python

import Buffy
import unittest

class BuffyTestConfig(unittest.TestCase):
	def setUp(self):
		self.config = Buffy.Config("test-config.txt")

	# Test Config
	def testConfig(self):
		self.failUnless(self.config)

		self.failIf(self.config.view().read())
		self.failIf(self.config.view().empty())
		self.failUnless(self.config.view().important())
		self.assertEqual(self.config.general().interval(), 600)

		locations = self.config.locations()
		self.assertEqual(locations.size(), 4)

		# This one seems not to have been wrapped, and I have no idea why
		programs = self.config.mailPrograms()
		self.assertEqual(programs.size(), 3)

		self.assertEqual(self.config.selectedMailProgram().name(), "mutt")

		self.config.selectMailProgram("Test")
		self.assertEqual(self.config.selectedMailProgram().name(), "Test");

		folder = Buffy.MailFolder.accessFolder("maildir/empty")
		self.failUnless(folder.valid())
		self.config.mailProgram("test").run(folder);

		folder = self.config.folder("foo")
		self.assertEqual(folder.forceview(), True)
		self.assertEqual(folder.forcehide(), False)
		folder.setForceHide(True)
		self.assertEqual(folder.forceview(), False)
		self.assertEqual(folder.forcehide(), True)

		folder = self.config.folder("bar")
		self.assertEqual(folder.forceview(), False)
		self.assertEqual(folder.forcehide(), True)
		folder.setForceView(True)
		self.assertEqual(folder.forceview(), True)
		self.assertEqual(folder.forcehide(), False)

class BuffyTestMaildir(unittest.TestCase):
	# Check an empty maildir
	def testEmpty(self):
		test = Buffy.MailFolder.accessFolder("maildir/empty")
		self.failUnless(test.valid())
		self.assertEqual(test.name(), "empty")
		self.assertEqual(test.path(), "maildir/empty")
		self.assertEqual(test.type(), "Maildir")

		self.assertEqual(test.getMsgTotal(), -1)
		self.assertEqual(test.getMsgUnread(), -1)
		self.assertEqual(test.getMsgNew(), -1)
		self.assertEqual(test.getMsgFlagged(), -1)
		self.failUnless(test.changed())

		test.updateStatistics()
		self.assertEqual(test.getMsgTotal(), 0)
		self.assertEqual(test.getMsgUnread(), 0)
		self.assertEqual(test.getMsgNew(), 0)
		self.assertEqual(test.getMsgFlagged(), 0)
		self.failIf(test.changed())

		folders = Buffy.MailFolder.enumerateFolders("maildir/empty")
		self.assertEqual(len(folders), 1)

	# Check a non empty maildir
	def testNonEmpty(self):
		test = Buffy.MailFolder.accessFolder("maildir/test")
		self.failUnless(test.valid())
		self.assertEqual(test.name(), "test")
		self.assertEqual(test.path(), "maildir/test")
		self.assertEqual(test.type(), "Maildir")

		self.assertEqual(test.getMsgTotal(), -1)
		self.assertEqual(test.getMsgUnread(), -1)
		self.assertEqual(test.getMsgNew(), -1)
		self.assertEqual(test.getMsgFlagged(), -1)
		self.failUnless(test.changed())

		test.updateStatistics()
		self.assertEqual(test.getMsgTotal(), 3)
		self.assertEqual(test.getMsgUnread(), 0)
		self.assertEqual(test.getMsgNew(), 0)
		self.assertEqual(test.getMsgFlagged(), 1)
		self.failIf(test.changed())

		folders = Buffy.MailFolder.enumerateFolders("maildir/test")
		self.assertEqual(len(folders), 1)

	# Check a maildir whose root is a broken symlink
	def test_broken(self):
		test = Buffy.MailFolder.accessFolder("maildir/broken")
		self.failIf(test.valid())

		folders = Buffy.MailFolder.enumerateFolders("maildir/broken")
		self.assertEqual(len(folders), 0)

	# Check a maildir whose cur, new and tmp directories are broken symlinks
	def testBroken1(self):
		test = Buffy.MailFolder.accessFolder("maildir/broken1")
		self.failIf(test.valid())

		folders = Buffy.MailFolder.enumerateFolders("maildir/broken1")
		self.assertEqual(len(folders), 0)

	# Check a maildir whose root is a symlink pointing to a nonexisting directory
	def testLoop(self):
		test = Buffy.MailFolder.accessFolder("maildir/loop")
		self.failIf(test.valid())

		folders = Buffy.MailFolder.enumerateFolders("maildir/loop")
		self.assertEqual(len(folders), 0)

	# Check a maildir which has a submaildir that is a symlink pointing up
	def testLoop1(self):
		test = Buffy.MailFolder.accessFolder("maildir/loop1")
		self.failUnless(test.valid())

		folders = Buffy.MailFolder.enumerateFolders("maildir/loop1")
		self.assertEqual(len(folders), 1)

	# Check maildir enumeration
	def testEnumeration(self):
		folders = Buffy.MailFolder.enumerateFolders("maildir")
		self.assertEqual(len(folders), 4)

class BuffyTestMailbox(unittest.TestCase):
	# Check an empty mailbox
	def test_empty(self):
		test = Buffy.MailFolder.accessFolder("mbox/empty.mbox")
		self.failUnless(test.valid())
		self.assertEqual(test.name(), "empty.mbox")
		self.assertEqual(test.path(), "mbox/empty.mbox")
		self.assertEqual(test.type(), "Mailbox")

		self.assertEqual(test.getMsgTotal(), -1)
		self.assertEqual(test.getMsgUnread(), -1)
		self.assertEqual(test.getMsgNew(), -1)
		self.assertEqual(test.getMsgFlagged(), -1)
		self.failUnless(test.changed())

		test.updateStatistics()
		self.assertEqual(test.getMsgTotal(), 0)
		self.assertEqual(test.getMsgUnread(), 0)
		self.assertEqual(test.getMsgNew(), 0)
		self.assertEqual(test.getMsgFlagged(), 0)
		self.failIf(test.changed())

		folders = Buffy.MailFolder.enumerateFolders("mbox/empty.mbox")
		self.assertEqual(len(folders), 1)

	# Check a non empty mailbox
	def test_nonempty(self):
		test = Buffy.MailFolder.accessFolder("mbox/test.mbox")
		self.failUnless(test.valid())
		self.assertEqual(test.name(), "test.mbox")
		self.assertEqual(test.path(), "mbox/test.mbox")
		self.assertEqual(test.type(), "Mailbox")

		self.assertEqual(test.getMsgTotal(), -1)
		self.assertEqual(test.getMsgUnread(), -1)
		self.assertEqual(test.getMsgNew(), -1)
		self.assertEqual(test.getMsgFlagged(), -1)
		self.failUnless(test.changed())

		test.updateStatistics()
		self.assertEqual(test.getMsgTotal(), 3)
		self.assertEqual(test.getMsgUnread(), 0)
		self.assertEqual(test.getMsgNew(), 0)
		self.assertEqual(test.getMsgFlagged(), 1)
		self.failIf(test.changed())

		folders = Buffy.MailFolder.enumerateFolders("mbox/test.mbox")
		self.assertEqual(len(folders), 1)

	# Check a mailbox whose root is a broken symlink
	def test_broken(self):
		test = Buffy.MailFolder.accessFolder("mbox/broken.mbox")
		self.failIf(test.valid())

		folders = Buffy.MailFolder.enumerateFolders("mbox/broken.mbox")
		self.assertEqual(len(folders), 0)

	# Check enumeration of a directory with mailboxes
	def test_enumeration(self):
		folders = Buffy.MailFolder.enumerateFolders("mbox")
		self.assertEqual(len(folders), 2)

if __name__ == "__main__":
	unittest.main()

## Read configuration
#conf = Buffy.Config()
#
## Decide if a mailbox is to be displayed
## Please note that all configuration methods also come with 'set_*' conterparts
#def view(x):
#	if x.getMsgUnread() > 0:
#		return True
#
#	if conf.view_important().get() and x.getMsgFlagged() > 0:
#		return True
#
#	if conf.view_read().get() and x.getMsgTotal() > 0:
#		return True
#
#	return conf.view_empty().get() or conf.folder(x.path()).get("forceview", "false").get() == "true"
#
#
## Get the folder objects from the paths in the config file
#folders = []
#for path in conf.folder_locations().get():
#	folders += Buffy.MailFolder_enumerateFolders(path)
#
##while True:
#	for f in folders:
##		print "Scanning " + f.name()
#		if f.changed():
#			f.updateStatistics()
#			if view(f):
#				print "%s: %d %d %d %d" % (f.name(), f.getMsgTotal(), f.getMsgUnread(), f.getMsgNew(), f.getMsgFlagged())
#	#time.sleep(conf.update_interval().get())





import Buffy, time

# Read configuration
conf = Buffy.Config()

# Decide if a mailbox is to be displayed
# Please note that all configuration methods also come with 'set_*' conterparts
def view(x):
	if x.getMsgUnread() > 0:
		return True

	if conf.view_important().get() and x.getMsgFlagged() > 0:
		return True

	if conf.view_read().get() and x.getMsgTotal() > 0:
		return True

	return conf.view_empty().get() or conf.folder(x.path()).get("forceview", "false").get() == "true"


# Get the folder objects from the paths in the config file
folders = []
for path in conf.folder_locations().get():
	folders += Buffy.MailFolder_enumerateFolders(path);

#while True:
	for f in folders:
#		print "Scanning " + f.name()
		if f.changed():
			f.updateStatistics()
			if view(f):
				print "%s: %d %d %d %d" % (f.name(), f.getMsgTotal(), f.getMsgUnread(), f.getMsgNew(), f.getMsgFlagged())
	#time.sleep(conf.update_interval().get())





#!/usr/bin/env python

"""
setup.py file for the libbuffy python bindings
"""

from distutils.core import setup, Extension
import distutils.command.build
import subprocess

# Code taken from http://code.activestate.com/recipes/502261-python-distutils-pkg-config/
# and then kind of fixed so that at least it is not offensive.
# It is however offensive that distutils doesn't do something as common as
# pkg-config by default. But then again, distutils is, as a whole, offensive.
def pkgconfig(*packages, **kw):
    flag_map = {'-I': 'include_dirs', '-L': 'library_dirs', '-l': 'libraries'}

    # Invoke pkg-config
    proc = subprocess.Popen(("pkg-config", "--libs", "--cflags") + packages,
            stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    out, err = proc.communicate()
    if proc.returncode != 0:
        raise RuntimeError("pkg-config returned error %d: %s" % (proc.returncode, err.strip()))

    # Enrich args with pkg-config output
    for token in out.split():
        arg = flag_map.get(token[:2], "extra_link_args")
        kw.setdefault(arg, []).append(token[2:])

    return kw

buffy_module = Extension('_Buffy', ['buffy.i'],
                         **pkgconfig('libbuffy >= 1.4', swig_opts=['-c++', '-I/usr/include'])
                        )

# distutils pretends it supports swig, but it obviously does it wrong (it is
# after all, as mentioned above, offensive). In this case, it looks for the
# swig-generated .py file before it runs swig to generate it. Doh!
#
# We therefore kick it by running build_ext first, so that Buffy.py is
# generated before build_py wants it.
#
# (sort is guaranteed stable since python 2.4, so we can depend on it)
distutils.command.build.build.sub_commands.sort(key=lambda x:x[0] != "build_ext")

setup(name = 'Buffy',
       version = '0.15',
       author      = "Enrico Zini <enrico@enricozini.org>",
       description = "libbuffy bindings for Python",
       ext_modules = [buffy_module],
       py_modules = ["Buffy"],
       )
